category_mapping = {
    "libs": "Libraries",
    "tools": "Tools",
    "compilers": "Compilers",
    "runtimes": "Runtimes",
    "": "",
    None: "",
}

group_mapping = {
    "ml": "Machine Learning and Computer Vision",
    "communication": "Communication",
    "math": "Math",
    "primitives": "Primitives",
    "dev": "Development",
    "perf": "Performance",
    "system": "System",
    "": "",
    None: "",
}
