/* Override PyData Sphinx Theme default colors */
html[data-theme='light'] {
    --pst-color-table-row-hover-bg: #E2E8F0;
}

html[data-theme='dark'] {
    --pst-color-table-row-hover-bg: #1E293B;
}

a svg {
  color: var(--pst-color-text-base);
}

a svg:hover {
  color: var(--pst-color-link-hover);
}

/* Adds container for big tables, used for Compatibility Matrix */
.format-big-table {
    white-space: nowrap;
}

/* Darken horizontal rules for better section separation */
hr {
  border: 0;
  border-top: 1px solid #999;
}